#!/bin/sh

#
# Author: Jamie Strandboge <jamie@canonical.com>
# Copyright (C) 2008 Canonical Ltd.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# http://www.gnu.org/copyleft/gpl.html
#

#
# USAGE:
# gen_certs.sh 512 1024 2048 ...
#
# Results:
# certs/cert-<bits>-<pid>.pem
# certs/key-<bits>-<pid>.pem
# certs/blacklist-$bits.db
#

getpid="./getpid.so"

if [ -z "$1" ]; then
    echo "Must supply a bit length"
    exit 1
fi

if [ ! -f "$getpid" ]; then
    echo "$getpid does not exist, exiting"
    exit 1
fi

mkdir certs 2> /dev/null || true

for bits in $@
do
    for type in nornd rnd
    do
        if [ "$type" = "nornd" ]; then
            # take ~/.rnd out of the picture to simulate first runs
            echo "Disabling ~/.rnd"
            rm -f ~/.rnd
            cat /dev/null > ~/.rnd
            chmod 0 ~/.rnd
            ls -l ~/.rnd
        else
            # seed ~/.rnd and make it writable for PID updates
            echo "Enabling ~/.rnd"
            rm -f ~/.rnd
            openssl req -newkey rsa:$bits -x509 -nodes -keyout certs/foo.pem -subj '/C=US/ST=Arizona/O=Testlib/OU=Test/CN=CA' -out certs/bar.pem
            rm -f certs/foo.pem certs/bar.pem
            ls -l ~/.rnd
        fi

        for i in $(seq 1 32768);
        do
            if [ -e "certs/cert-$bits-$i-$type" ]; then
                continue
            fi
            FORCE_PID=$i LD_PRELOAD="$getpid" openssl req -newkey rsa:${bits} -x509 -nodes -keyout certs/key-$bits-$i-$type.pem -subj '/C=US/ST=Arizona/O=Testlib/OU=Test/CN=CA' -out certs/cert-$bits-$i-$type.pem
            openssl rsa -noout -modulus -in certs/key-$bits-$i-$type.pem | sha1sum | cut -d ' ' -f 1 >> certs/blacklist-${bits}.db
        done
    done
done

